package com.chelvc.cloud.maintain.controller;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;
import javax.validation.Valid;
import javax.validation.constraints.Min;

import com.chelvc.cloud.maintain.copier.MerchantCopier;
import com.chelvc.cloud.maintain.copier.ReservationCopier;
import com.chelvc.cloud.maintain.vo.ReservationVO;
import com.chelvc.cloud.vehicle.client.ReservationClient;
import com.chelvc.cloud.vehicle.client.dto.MerchantReservationDTO;
import com.chelvc.cloud.vehicle.client.dto.ReservationDTO;
import com.chelvc.cloud.vehicle.client.param.ReservationModifyParam;
import com.chelvc.cloud.vehicle.client.param.ReservationPagingParam;
import com.chelvc.cloud.vehicle.client.param.ReservationQueryParam;
import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.common.model.Pagination;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * 预约接口
 *
 * @author Woody
 * @date 2023/7/17
 */
@Validated
@RestController
@ResponseWrapping
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class ReservationController {
    private final ReservationClient reservationClient;

    /**
     * 新增预约
     *
     * @param merchantId 商家ID
     * @param param      预约参数
     */
    @PostMapping("/merchant/{merchantId}/reservation")
    public void addReservation(@PathVariable("merchantId") @Min(value = 1, message = "商家主键不能小于1") Long merchantId,
                               @RequestBody @Valid ReservationModifyParam param) {
        this.reservationClient.addReservation(merchantId, param);
    }

    /**
     * 查询用户预约列表
     *
     * @param param 查询参数
     * @return 用户预约列表
     */
    @GetMapping("/reservations")
    public List<ReservationVO> listUserReservations(@Valid ReservationQueryParam param) {
        List<ReservationDTO> reservations = this.reservationClient.listUserReservations(param);
        if (CollectionUtils.isEmpty(reservations)) {
            return Collections.emptyList();
        }
        return reservations
                .stream()
                .map(o -> {
                    ReservationVO reservationVO = ReservationCopier.INSTANCE.copying(o);
                    reservationVO.setSimpleMerchantVO(MerchantCopier.INSTANCE
                            .merchantDTOToSimpleMerchantVO(o.getMerchant()));
                    return reservationVO;
                }).collect(Collectors.toList());
    }

    /**
     * 取消预约
     *
     * @param id 预约主键
     */
    @PutMapping("/reservation/{id}")
    public void deleteReservation(@PathVariable("id") @Min(value = 1, message = "预约主键不能小于1") Long id) {
        this.reservationClient.deleteReservation(id);
    }


    /**
     * 查询商家被预约列表
     *
     * @param param 查询参数
     * @return 商家被预约列表
     */
    @PostMapping("/listMerchantReservations")
    public List<ReservationVO> listMerchantReservations(@RequestBody @Valid ReservationQueryParam param) {
        List<ReservationDTO> reservations = this.reservationClient.listMerchantReservations(param);
        if (CollectionUtils.isEmpty(reservations)) {
            return Collections.emptyList();
        }
        return reservations
                .stream()
                .map(o -> {
                    ReservationVO reservationVO = ReservationCopier.INSTANCE.copying(o);
                    reservationVO.setSimpleMerchantVO(MerchantCopier.INSTANCE
                            .merchantDTOToSimpleMerchantVO(o.getMerchant()));
                    return reservationVO;
                }).collect(Collectors.toList());
    }


    /**
     * 商家端分页查询预约
     *
     * @param param 查询参数
     * @return 预约分页信息
     */
    @GetMapping("/reservation/getMerchantList")
    public Pagination<MerchantReservationDTO> getMerchantReservationPaging(@Valid ReservationPagingParam param) {
        return this.reservationClient.getMerchantReservationPaging(param);
    }
}
