package com.chelvc.cloud.maintain.controller;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;
import javax.validation.Valid;
import javax.validation.constraints.Min;

import com.chelvc.cloud.maintain.copier.UserCouponCopier;
import com.chelvc.cloud.maintain.vo.UserCouponVO;
import com.chelvc.cloud.vehicle.client.UserCouponClient;
import com.chelvc.cloud.vehicle.client.dto.UserCouponDTO;
import com.chelvc.cloud.vehicle.client.param.UserCouponPagingParam;
import com.chelvc.cloud.vehicle.client.param.UserCouponQueryParam;
import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.common.model.Pagination;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 优惠券领取记录接口
 *
 * @author qizai
 * @Date 2023/9/8
 **/
@Validated
@RestController
@ResponseWrapping
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class UserCouponController {
    private final UserCouponClient userCouponClient;

    /**
     * 领取优惠券
     *
     * @param couponId 优惠券ID
     * @return 优惠券领取记录主键
     */
    @PostMapping("/usercoupon/{couponId}/claim")
    public Long claimCoupon(@PathVariable("couponId") @Min(value = 1, message = "优惠券主键不能小于1") Long couponId) {
        return this.userCouponClient.claimCoupon(couponId);
    }

    /**
     * 自动领取优惠券
     */
    @PostMapping("/usercoupon/autoclaim")
    public void autoClaimCoupon() {

    }

    /**
     * 获取用户领取的优惠券列表
     *
     * @param param 查询参数
     * @return 用户已领取的优惠券信息列表
     */
    @GetMapping("/usercoupons")
    public List<UserCouponVO> listUserCoupons(@Valid UserCouponQueryParam param) {
        List<UserCouponDTO> userCoupons = this.userCouponClient.listUserCoupons(param);
        if (CollectionUtils.isEmpty(userCoupons)) {
            return Collections.emptyList();
        }
        return userCoupons
                .stream()
                .map(o -> UserCouponCopier.INSTANCE.copying(o, o.getCouponDTO()))
                .collect(Collectors.toList());
    }

    /**
     * 获取当前用户对于当前商品可使用的优惠券列表
     *
     * @param param 查询参数
     * @return 用户可用优惠券信息列表
     */
    @GetMapping("/canusecoupons")
    public List<UserCouponVO> listUserCanUseCoupons(@Valid UserCouponQueryParam param) {
        List<UserCouponDTO> userCoupons = this.userCouponClient.listUserCanUseCoupons(param);
        if (CollectionUtils.isEmpty(userCoupons)) {
            return Collections.emptyList();
        }
        return userCoupons
                .stream()
                .map(o -> UserCouponCopier.INSTANCE.copying(o, o.getCouponDTO()))
                .collect(Collectors.toList());
    }

    /**
     * 获取优惠券领取记录信息
     *
     * @param id 优惠券领取记录主键
     * @return 优惠券领取记录信息
     */
    @GetMapping("/usercoupon/{id}")
    public UserCouponVO getUserCoupon(@PathVariable("id") @Min(value = 1, message = "优惠券领取记录主键不能小于1") Long id) {
        return UserCouponCopier.INSTANCE.copying(this.userCouponClient.getUserCoupon(id));
    }

    /**
     * 查询优惠券领取记录分页
     *
     * @param param 查询参数
     * @return 优惠券领取记录分页信息
     */
    @GetMapping("/usercoupon/paging")
    public Pagination<UserCouponVO> getUserCouponPaging(@Valid UserCouponPagingParam param) {
        Pagination<UserCouponDTO> pagination = this.userCouponClient.getUserCouponPaging(param);
        List<UserCouponDTO> records = pagination.getRecords();
        if (CollectionUtils.isEmpty(records)) {
            return Pagination.empty();
        }

        // 构建优惠券领取记录信息
        return pagination.convert(UserCouponCopier.INSTANCE::copying);
    }
}
